function [vnet,SV,conc]=ElementaryRxnsODE(Kvec,Econc,Tconc,Sp,Ae,Info_m,Vcof_index,refMetab,refPro,maxtime_,MetabFactor)

%for using the inputs from "getExpandedA_2" and "getExpandedS_2" functions
%which model the transporters of the pathway substrates having MM kinetics
%
%consider the moiety conservation in initial concentration.  Vcof_index
%provides indices of either one of the cofactors in the pairs (such taht
%either ATP or ADP)
%
%-------------------------------
%Linh Tran, UCLA (& Matt)
%Last revision June 25, 2008
%-------------------------------

tmax = maxtime_; 
step_=maxtime_/100;
niter=1;

nin_=length(find(Info_m(:,7)==2));
nout_=length(find(Info_m(:,7)==3));
indInterEnzID=find(Info_m(:,7)==1);
nInterEnz=length(unique(Info_m(indInterEnzID,2)));  %might be different from number of rxns
nInterRxn=length(find(Info_m(:,7)==1));
nRevRxns=nInterRxn+nin_;
nNetRxns=nRevRxns+nout_;
totalRxns=size(Info_m,1);

%Find how namy complexes and free enzyme
if nNetRxns<totalRxns
    nEnzComp_=Info_m(totalRxns,6);
else
    nEnzComp_=Info_m(nRevRxns,6);
end
nRevEnz=max(Info_m(1:nRevRxns,2));  %no transporters for Vout
nMetab=size(Sp,1)-nEnzComp_;
nvuni_=size(Sp,2);

%%  Initialize the initial condition for ODE by setting
%%(1)all reversible enzymes and their intermediates: substrates
%%(2)transporters for outfluxes: included in k values
xo_=repmat(1,nin_,1);  %concentrations of the pathway substrates, which are constant and included into k values
EnzComp_=zeros(nEnzComp_,1); 
Ke=Kvec;

%initialize the metabolite concentrations
if isempty(refMetab)  
    refMetab=repmat(1,nMetab,1);
end
iniMetab=refMetab;

%initialize the enzyme concentrations
if isempty(refPro)
    flag_=1;    
else
    flag_=0;
    EnzComp_=refPro(1:nEnzComp_,1);
end

%Find the conservation pattern of cofactors
Nl=null(Sp','r');
nCofactors=length(Vcof_index);
allSel=[];
for jj=1:nCofactors
    seltmp=find(Nl(Vcof_index(jj),:)==-1);
    allSel=[allSel;seltmp'];
end
if length(allSel)>nCofactors
    allSel_new=unique(allSel);
    tmpCofac=Nl(Vcof_index,allSel_new);
    Xalpha=tmpCofac\(-eye(nCofactors));
    tmpCofac=[];
    tmpCofac=Nl(:,allSel_new)*Xalpha;
    Nl(:,allSel_new)=tmpCofac;
end
selCol=[];
for  jj=1:nCofactors
    selCol(jj)=find(Nl(Vcof_index(jj),:)==-1);
end
blockPatt=Nl(nMetab+1:end,selCol);
bookKeeping_=zeros(nRevEnz,nCofactors);
%Initialize enzymes
for k=1:nInterEnz
    vrxns=find(Info_m(:,2)==k); %find all including inhibition
    ind_tmp=k;
    for kk=1:length(vrxns)
        if Info_m(vrxns(kk),7)~=3  %not an Vout
            ind_tmp=[ind_tmp,Info_m(vrxns(kk),5):1:Info_m(vrxns(kk),6)]; %row vector
        end
    end
    
    if flag_
        tmp_nor=rand(length(ind_tmp),1);
        tmp_nor=tmp_nor/sum(tmp_nor);
        EnzComp_(ind_tmp,1)=tmp_nor;
        tmp_E=1;
    else
        tmp_E=sum(EnzComp_(ind_tmp,1));
    end
    delE_=Econc(k,1)-tmp_E;  %free enzyme
    if delE_>0  %overexpress
        EnzComp_(k,1)=EnzComp_(k,1)+delE_;  %increase the free enzyme
    else   %underexpress
        EnzComp_(ind_tmp,1)=EnzComp_(ind_tmp,1)*Econc(k,1)/tmp_E;  %decrease each form by factor (total new/total old)
        bookKeeping_(k,:)=(1-tmp_E/Econc(k,1))*EnzComp_(ind_tmp,1)'*blockPatt(ind_tmp,:);
        for cc=1:nCofactors
            if bookKeeping_(k,cc)
                bookKeeping_(k,cc)=bookKeeping_(k,cc)-delE_;
            end
        end
    end
end
%Initialize transporters of influxes
for k=nInterEnz+1:1:nInterEnz+nin_ 
    vrxns=find(Info_m(:,2)==k); %find all including inhibition
    rxnID=unique(Info_m(vrxns,1));
    if length(rxnID)>1
        error('\nPlease use separate transporter for different external metabolite');
    end
    ind_tmp=k;
    for kk=1:length(vrxns)
        if Info_m(vrxns(kk),7)~=3  %not an Vout
            ind_tmp=[ind_tmp,Info_m(vrxns(kk),5):1:Info_m(vrxns(kk),6)]; %row vector
        end
    end
    if flag_
        tmp_nor=rand(length(ind_tmp),1);
        tmp_nor=tmp_nor/sum(tmp_nor);
        EnzComp_(ind_tmp,1)=tmp_nor;        
        tmp_E=1;
    else
        tmp_E=sum(EnzComp_(ind_tmp,1,1));
    end
    delE_=Tconc(k-nInterEnz,1)-tmp_E;
    if delE_>0  %overexpress
        EnzComp_(k,1)=EnzComp_(k,1)+delE_;  %increase the free enzyme
    else   %underexpress
        EnzComp_(ind_tmp,1)=EnzComp_(ind_tmp,1)*Tconc(k-nInterEnz,1)/tmp_E;  %decrease each form by factor (total new/total old)
        bookKeeping_(k,:)=(1-tmp_E/Tconc(k-nInterEnz,1))*EnzComp_(ind_tmp,1)'*blockPatt(ind_tmp,:);
        for cc=1:nCofactors
            if bookKeeping_(k,cc)
                bookKeeping_(k,cc)=bookKeeping_(k,cc)-delE_;
            end
        end
    end
    Ke(Info_m(rxnID,3),1)=xo_(k-nInterEnz)*Ke(Info_m(rxnID,3),1);
end

Vout_index=find(Info_m(:,7)==3);
%Check the input
if length(Vout_index)~=length(Tconc)-nin_
    error('\nPlease check the number of transporters');
end
Ke(Info_m(Vout_index,3),1)=Ke(Info_m(Vout_index,3),1).*Tconc(nin_+1:end);

if ~isempty(Vcof_index)
    iniMetab(Vcof_index)=refMetab(Vcof_index)+sum(bookKeeping_)';
end

%Initial concentrations
initial_=[iniMetab' EnzComp_'];   

[t,conc] = ode15s(@metabODE2,[0:step_:tmax], initial_,[],Ke,Ae,Sp,refMetab,MetabFactor);

rate_=rateCalc(conc,Ke,Ae);

vuni=rate_(:,end);
SV = Sp*vuni;

ind_for1=Info_m(1:nRevRxns,3);
ind_rev1=ind_for1+1;
vnet=vuni(ind_for1,1)-vuni(ind_rev1,1);
vnet=[vnet;vuni(Info_m(nRevRxns+1,3):Info_m(nRevRxns+nout_,3))];

%-----------------------------------------------------------
function dx = metabODE2(t,x,Ke,Ae,Sp,refMetab,MetabFactor)

[nrxns,nsubs]=size(Ae);

dx = zeros(nsubs,1);

V=zeros(nrxns,1);
log_x=log(x);
tmp=sign(Ae)*log_x;
V=Ke.*exp(tmp);
n=length(refMetab);
dx=Sp*V;
dx(1:n)=dx(1:n)/MetabFactor;

%----------------------------------------------
function rate_=rateCalc(xconc,Ke,Ae)
    
ndata=size(xconc,1);
[nrxns,nsubs]=size(Ae);
rate_=zeros(nrxns,ndata);
log_x=log(xconc)';
tmp=sign(Ae)*log_x;
for k=1:ndata
    rate_(:,k)=Ke.*exp(tmp(:,k));
end


