clear
load branch
rand('twister',sum(100*clock))

%% input the stoichiometric matrix along with required indices and fluxes
S=Net.S;
Sreg=Net.Sreg;
EnzName=Net.EnzName;
Vin_index=Net.Vin_index;
Vout_index=Net.Vout_index;
Vcof_index=Net.Vcof_index;
% input the thermodynamic constraints
SGFEnergy=Net.SGFE;
MetabRange=Net.MetabRange;
%Input the model-specific values
ngens=Net.Models;  %Number of Models
fold = Net.Fold; % Fold Change
maxtime_=Net.Time;   % Time
initmetab_=ones(size(Net.S(:,1))); %Number of Metabolites
V_out=Net.ProductFlux; %Product Flux
V_in=Net.UptakeFlux; %Uptake Flux
perturb = Net.Perturb; %Enzymes to Perturb
converge = Net.Convergence; %Value to indicate convergence
%Rearrange reactions - reactions catalyzed by same enzyme
totalNetRxns=size(S,2);
nin_=length(Vin_index);
nout_=length(Vout_index);
nInRxns=totalNetRxns-nin_-nout_;
[newOrder,ind_]=sort(EnzName(1:nInRxns));
S(:,1:nInRxns)=S(:,ind_);
if ~isempty(Sreg)
    Sreg(:,1:nInRxns)=Sreg(:,ind_);
end
EnzName(1:nInRxns)=newOrder;
SGFEnergy(1:nInRxns)=SGFEnergy(ind_);
[junk1,junk2,EnzID]=unique(EnzName(1:nInRxns));  %make sure no gap in EnzID
tmp=[length(junk1)+1:1:length(junk1)+nin_+nout_]; %#ok<NBRAK>
EnzID=[EnzID;tmp'];
%% Calculate the range of free energies if it is required
rt=1.987*298/1000;
SGFEnergy=SGFEnergy/rt;
negS=S;
posS=S;
v=find(posS<0);
posS(v)=0; %#ok<FNDSB>
v=find(negS>0);
negS(v)=0;
lnQ_low=negS'*log(MetabRange(:,2))+posS'*log(MetabRange(:,1));
lnQ_high=negS'*log(MetabRange(:,1))+posS'*log(MetabRange(:,2));
FreeE_low=SGFEnergy+lnQ_low;
FreeE_high=SGFEnergy+lnQ_high;
%% Get the S and A matrices for MM model
[Asub,Aenz,Info_m]=getExpandedA(S,Sreg,Vin_index,Vout_index,EnzID);
[Spsub,Spenz]=getExpandedS(S,Sreg,Vin_index,Vout_index,EnzID);
clear S Sreg EnzID EnzName SGFEnergy MetabRange Vin_index Vout_index negS posS v lnQ* ind_ junk1 junk2 newOrder rt

%% Parameters relating to the size of the network
[nEnzComp_, nvuni]=size(Spenz);  
nMetab_=size(Asub,2);
nPro_=size(Aenz,2);
rVnet = Net.rVnet;

% Thermodynamic constraints
FElow_new=FreeE_low;
FEhigh_new=FreeE_high;
for k=1:nInRxns
    if FreeE_low(k)>0 && rVnet(k)>0
        fprintf('\nThe direction of the reaction %i is opposite to the thermo constraint',k);
    end
    if FreeE_low(k)<0 && FreeE_high(k)>0
        if rVnet(k)>0
            FEhigh_new(k)=0;
        else
            FElow_new(k)=0;
        end
    end
    if FreeE_high(k)<0 && rVnet(k)<0
        fprintf('\nThe direction of the reaction %i is opposite to the thermo constraint',k);
    end
end
Sp=[Spsub;Spenz];
Ae_ode=[Asub,Aenz(:,1:nEnzComp_)];
clear Spsub Spenz FreeE_high FreeE_low

ssmetab_=ones(size(initmetab_));
BigVnet_r=[];
BigVnet_oe=[];
BigKvec_r=[];
BigPro_r=[];
%BigConc_r = [];
%BigConc_oe = [];
bad_value = [];
maxtime_old = maxtime_;
iteration = 1;
iteration_old = iteration;
MetabFactor =10; %Factor to scale metabolite concentrations

for ngen=1:ngens
    Rref=samplingR(Info_m,FElow_new,FEhigh_new);
    [Kvec,Econc,Tconc,pro_]=getKineticPara(rVnet,Info_m,Asub,Aenz,Rref,ssmetab_);
    %Kvec = BigKvec_r(:,ngen);
    %pro_ = BigPro_r(:,ngen);
    [vnet,SV,conc]=ElementaryRxnsODE(Kvec,Econc,Tconc,Sp,Ae_ode,Info_m,Vcof_index,initmetab_,pro_,maxtime_,MetabFactor);
    while max(abs(SV))>converge || ~isreal(vnet) ||norm(rVnet-vnet) > 0.001
        Rref=samplingR(Info_m,FElow_new,FEhigh_new);
        [Kvec,Econc,Tconc,pro_]=getKineticPara(rVnet,Info_m,Asub,Aenz,Rref,ssmetab_);
        [vnet,SV,conc]=ElementaryRxnsODE(Kvec,Econc,Tconc,Sp,Ae_ode,Info_m,Vcof_index,initmetab_,pro_,maxtime_,MetabFactor);
    end
    BigVnet_r(:,ngen)=vnet;
    % BigConc_r(:,ngen)=conc(end,:)';
    BigKvec_r(:,ngen)=Kvec;
    BigPro_r(:,ngen)=pro_;
    for j = perturb
        k = find(perturb==j);
        Econc_old = Econc;
        Econc(j) = fold*Econc(j);
        [vnet2,SV2,conc2]=ElementaryRxnsODE(Kvec,Econc,Tconc,Sp,Ae_ode,Info_m,Vcof_index,initmetab_,pro_,maxtime_,MetabFactor);
        if ~isreal(vnet2) || max(vnet2)<1e-3
            BigVnet_oe(:,ngen,k)=NaN(length(rVnet),1);
            %BigConc_oe(:,ngen,k)=NaN(length(SV),1);
            %bad_value = [bad_value; j ngen];
            maxtime_ = maxtime_old;
            iteration = iteration_old;
            Econc = Econc_old;
        else
            while max(abs(SV2))>converge && iteration <5
                maxtime_ = 2*maxtime_;
                [vnet2,SV2,conc2]=ElementaryRxnsODE(Kvec,Econc,Tconc,Sp,Ae_ode,Info_m,Vcof_index,initmetab_,pro_,maxtime_,MetabFactor);
                if iteration == 4
                    BigVnet_oe(:,ngen,k)=NaN(length(rVnet),1);
                    %BigConc_oe(:,ngen,k)=NaN(length(SV),1);
                    %bad_value = [bad_value; j ngen];
                elseif ~isreal(vnet2) || max(vnet2)<1e-3
                    BigVnet_oe(:,ngen,k)=NaN(length(rVnet),1);
                    %BigConc_oe(:,ngen,k)=NaN(length(SV),1);
                    %bad_value = [bad_value; j ngen];
                    iteration = 4;
                end
                iteration = iteration + 1;
            end
            if iteration == 5
                maxtime_ = maxtime_old;
                iteration = iteration_old;
                Econc = Econc_old;
            else
                maxtime_ = maxtime_old;
                iteration = iteration_old;
                Econc = Econc_old;
                BigVnet_oe(:,ngen,k)=vnet2;
                %BigConc_oe(:,ngen,k)=conc2(end,:)';
            end
        end
        save ('results_branch')
    end
    ngen %#ok<NOPTS>
end
BigVnet_in = squeeze(BigVnet_oe(V_in,:,:));
BigVnet_out = squeeze(BigVnet_oe(V_out,:,:));

clear Info_m A* Sp FE* Kvec MetabFactor Rref V_in V_out Vcof_index fold rVnet pro_ initmetab_ ssmetab_ iteration* j k maxtime_* n* perturb* tmp totalNetRxns

save ('results_branch')