function [Spsub,Spenz]=getExpandedS(S,Sreg,Vin_index,Vout_index,EnzID)

%Converting the stoichiometric coefficient matrix of overall reactions into
%the corresponding stoichiometric coefficient matrix of elementary
%reactions.  The transporters corresponding to the input and output must be
%last columns.  The influx is going to be reversible, while the outflux(es)
%is irreversible
%
%The transporters of pathway substrates are also have MM kinetics
%
%Incorporating the inhibition
%--------------------------------------
%Last revision Oct 31, 2007
%Linh Tran, UCLA
%Matthew Rizk, UCLA
%--------------------------------------

[nMetab,nNetRxns]=size(S);

if isempty(EnzID)  %no input for indices of enzymes catalyzing the rxns
    EnzID=[1:1:nNetRxns]';  %different enzyme catalyzes different rxn  
end

nin_=length(Vin_index);
nout_=length(Vout_index);
nInterRxns=nNetRxns-nin_-nout_;
nRevRxns=nInterRxns+nin_;
totalRevEnz=length(unique(EnzID(1:nRevRxns)));

vtmp=find(S>0);
Spro=zeros(nMetab,nNetRxns);
Spro(vtmp)=S(vtmp);

vtmp=find(S<0);
Ssub=zeros(nMetab,nNetRxns);
Ssub(vtmp)=S(vtmp);

vnsub=sum(Ssub);
vnpro=sum(Spro);
rcount=1+totalRevEnz;
ccount=1;
Spenz=[];
Spsub=[];
HKmatrix=[];  %start indices of intermediates involved to the rxn

for k=1:nInterRxns
    HKmatrix(k,1)=rcount;
    if vnsub(k)==-2 & vnpro(k)==2
    %two substrates and two products: 4 steps,4 enzyme complexes, and 8 Vuni 
        blockSsub=zeros(nMetab,8);
        subID=getExpandedIndices(Ssub(:,k));
        blockSsub(subID(1),1)=-1;
        blockSsub(subID(2),3)=-1;
        blockSsub(subID(1),2)=1;
        blockSsub(subID(2),4)=1;            

        proID=getExpandedIndices(Spro(:,k)); 
        blockSsub(proID(1),6)=-1;
        blockSsub(proID(2),8)=-1;
        blockSsub(proID(1),5)=1;
        blockSsub(proID(2),7)=1;
                        
        blockSenz=zeros(4,8);
        blockSenz(1,[1 8])=-1;
        blockSenz(2,2:3)=-1; 
        blockSenz(3,4:5)=-1;
        blockSenz(4,6:7)=-1;
        blockSenz(1,[2 7])=1;
        blockSenz(2,[1 4])=1; 
        blockSenz(3,[3 6])=1;
        blockSenz(4,[5 8])=1;
    else
        blockSenz=zeros(3,6);
        blockSenz(1,[1 6])=-1;
        blockSenz(2,2:3)=-1; 
        blockSenz(3,4:5)=-1;
        blockSenz(1,[2 5])=1;
        blockSenz(2,[1 4])=1; 
        blockSenz(3,[3 6])=1;
       
        blockSsub=zeros(nMetab,6);
        subID=getExpandedIndices(Ssub(:,k));
        if vnsub(k)==-2 & vnpro(k)==1
        %2 substrates and 1 products: 3 steps,3 enzyme complexes, and 6 Vuni    
            blockSsub(subID(1),1)=-1;
            blockSsub(subID(2),3)=-1;
            blockSsub(subID(1),2)=1;
            blockSsub(subID(2),4)=1;

            proID=getExpandedIndices(Spro(:,k));
            blockSsub(proID,6)=-1;
            blockSsub(proID,5)=1;
        else  %1 substrates        
            blockSsub(subID,1)=-1;
            blockSsub(subID,2)=1;
            proID=getExpandedIndices(Spro(:,k));
            if vnpro(k)==1  %1 product
                blockSsub(proID,6)=-1;
                blockSsub(proID,5)=1;
            else             %2 products
                blockSsub(proID(1),4)=-1;
                blockSsub(proID(2),6)=-1;
                blockSsub(proID(1),3)=1;
                blockSsub(proID(2),5)=1;                
            end
        end
    end
    if vnsub(k)==-2 & vnpro(k)==3
        %two substrates and three products: 5 steps,5 enzyme complexes, and 10 Vuni
        blockSsub=zeros(nMetab,10);
        subID=getExpandedIndices(Ssub(:,k));
        blockSsub(subID(1),1)=-1;
        blockSsub(subID(2),3)=-1;
        blockSsub(subID(1),2)=1;
        blockSsub(subID(2),4)=1;

        proID=getExpandedIndices(Spro(:,k));
        blockSsub(proID(1),6)=-1;
        blockSsub(proID(2),8)=-1;
        blockSsub(proID(3),10)=-1;
        blockSsub(proID(1),5)=1;
        blockSsub(proID(2),7)=1;
        blockSsub(proID(3),9)=1;

        blockSenz=zeros(5,10);
        blockSenz(1,[1 10])=-1;
        blockSenz(2,2:3)=-1;
        blockSenz(3,4:5)=-1;
        blockSenz(4,6:7)=-1;
        blockSenz(5,8:9)=-1;
        blockSenz(1,[2 9])=1;
        blockSenz(2,[1 4])=1;
        blockSenz(3,[3 6])=1;
        blockSenz(4,[5 8])=1;
        blockSenz(5,[7 10])=1;
    end

    if vnsub(k)<-2 || vnpro(k)>3
        error('\nThe program limits to maximum of 2 reactants and 3 products per reaction\n');
    end   
    
    Spsub=[Spsub,blockSsub];
    [nr,ne]=size(blockSenz);
    Spenz(EnzID(k),ccount:ccount+ne-1)=blockSenz(1,:);
    Spenz(rcount:rcount+nr-2,ccount:ccount+ne-1)=blockSenz(2:nr,:);
    rcount=rcount+nr-1;
    ccount=ccount+ne;
end             

for k=1:nin_ 
    HKmatrix(k+nInterRxns,1)=rcount;
    
    blockSsub=zeros(nMetab,6);
    subID=getExpandedIndices(Ssub(:,nInterRxns+k));
    blockSsub(subID,1)=-1;
    blockSsub(subID,2)=1;
    proID=getExpandedIndices(Spro(:,nInterRxns+k));
    if vnpro(nInterRxns+k)==1  %1 product
        blockSsub(proID,6)=-1;
        blockSsub(proID,5)=1;
    else             %2 products
        blockSsub(proID(1),4)=-1;
        blockSsub(proID(2),6)=-1;
        blockSsub(proID(1),3)=1;
        blockSsub(proID(2),5)=1;                
    end
        
    blockSenz=zeros(3,6);
    blockSenz(1,[1 6])=-1;
    blockSenz(2,2:3)=-1; 
    blockSenz(3,4:5)=-1;
    blockSenz(1,[2 5])=1;
    blockSenz(2,[1 4])=1; 
    blockSenz(3,[3 6])=1;
    
    Spsub=[Spsub,blockSsub];
    [nr,ne]=size(blockSenz);
    Spenz(EnzID(nInterRxns+k),ccount:ccount+ne-1)=blockSenz(1,:);
    Spenz(rcount:rcount+nr-2,ccount:ccount+ne-1)=blockSenz(2:nr,:); 
    rcount=rcount+nr-1;
    ccount=ccount+ne;
end

nTinEnz_=size(Spenz,1);
for k=1:nout_
    subID=find(Ssub(:,Vout_index(k)));
    blockSsub=zeros(nMetab,1);  %assume both ref and target state have the same concentration of pathway substrate(s)
    blockSsub(subID,1)=-1;   
    
    blockSenz=zeros(nTinEnz_,1);
    
    Spsub=[Spsub,blockSsub];
    Spenz=[Spenz,blockSenz];  %add column corresponding to V, but not row for enz
    ccount=ccount+1;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%FOR INHIBITION REGULATION
if ~isempty(Sreg)
    [xinh,yinh]=find(Sreg<0);
    vtmp=find(Sreg<0);

    for j=1:length(vtmp)
        located_=[];
        nNormalEnz=rcount-1;
        if Sreg(vtmp(j))==-1
            %competitive inhibition:  1 step, 1 enzyme complex, and 2 Vuni
            blockSinhsub=zeros(nMetab,2);
            blockSinhsub(xinh(j),1)=-1;
            blockSinhsub(xinh(j),2)=1;

            blockSinhenz1=zeros(nNormalEnz,2);
            located_=EnzID(yinh(j));
            blockSinhenz1(located_,:)=[-1 1];
            
            blockSinhenz2=[1 -1];
         
        elseif Sreg(vtmp(j))==-2
            %uncompetitive inhibition:  1 step, 1 enzyme complex, and 2 Vuni
            blockSinhsub=zeros(nMetab,2);
            blockSinhsub(xinh(j),1)=-1;
            blockSinhsub(xinh(j),2)=1;
            
            blockSinhenz1=zeros(nNormalEnz,2);
            located_=HKmatrix(yinh(j));
            blockSinhenz1(located_,:)=[-1 1];
            
            blockSinhenz2=[1 -1];
         
        elseif Sreg(vtmp(j))==-3
            %mixed inhibition:  2 steps, 2 enzyme complexes, and 4 Vuni
            blockSinhsub=zeros(nMetab,4);
            blockSinhsub(xinh(j),[1 3])=-1;
            blockSinhsub(xinh(j),[2 4])=1;
                     
            blockSinhenz1=zeros(nNormalEnz,4);
            located_(1)=EnzID(yinh(j));
            located_(2)=HKmatrix(yinh(j));
            blockSinhenz1(located_(1),1:2)=[-1 1];
            blockSinhenz1(located_(2),3:4)=[-1 1];
       
            blockSinhenz2=zeros(2,4);
            blockSinhenz2(1,1:2)=[1 -1];
            blockSinhenz2(2,3:4)=[1 -1];
        else
            error('\nThe program only accepts inhibition labeled with -1 or -2 or -3\n');
        end
        Spsub=[Spsub,blockSinhsub];
        [nr1,ne1]=size(blockSinhenz1);
        [nr2,ne2]=size(blockSinhenz2);
        Spenz=[Spenz,blockSinhenz1];
        Spenz(rcount:rcount+nr2-1,ccount:ccount+ne2-1)=blockSinhenz2;
        rcount=rcount+nr2;
        ccount=ccount+ne2;
    end
end

%------------------------------------------------------
function v_ind=getExpandedIndices(v_input)

n=length(v_input);
v_input=abs(v_input);
v_ind=[];
for k=1:n
    if v_input(k)==1
        v_ind=[v_ind;k];
    elseif v_input(k)>1
        v_ind=[v_ind;repmat(k,v_input(k),1)];
    end
end