function [Kvec,Econc,Tconc,Pro_]=getKineticPara(rVnet,Info_m,Asub,Aenz,Rref,ssmetab_)
%function [Kvec,Econc,Tconc,rVuni,Metab_,Pro_]=getKineticPara(rVnet,Info_m,Asub,Aenz,Rref,ssmetab_)

%get kinetic parameters, concentrations of enzymes and transporters,
%reversibility, and Vuni from Vnet
%
%for using the inputs from "getExpandedA_2" and "getExpandedS_2" functions
%which model the transporters of the pathway substrates having MM kinetics
%
%considering the Thermodynamics constraint in sampling R
%
%Fix the bug of mismatching indices when transporters are normalized
%-------------------------------
%Linh Tran, UCLA
%Last revision April 21, 2008
%-------------------------------
thres_=0.4;  %ratio between binding reactions of inhibitor and substrate to enzyme

nMetab_=size(Asub,2);
nPro_=size(Aenz,2);


totalRxns=size(Info_m,1);  %inclusing the inhibition reactions
nvuni_=Info_m(totalRxns,4);

nin_=length(find(Info_m(:,7)==2));
nout_=length(find(Info_m(:,7)==3));
nInterRxns=length(find(Info_m(:,7)==1));
nRevRxns=nInterRxns+nin_;

Metab_= ssmetab_; 

Pro_=rand(nPro_,1);  
nRevEnz=max(Info_m(1:nRevRxns,2));  %no Vout
for k=1:nRevEnz
    vrxns=find(Info_m(:,2)==k);
    ind_tmp=k;
    for kk=1:length(vrxns)
        if Info_m(vrxns(kk),7)~=3  %not an outflux transporter
            ind_tmp=[ind_tmp,Info_m(vrxns(kk),5):1:Info_m(vrxns(kk),6)]; %row vector
        end
    end
    tmp_nor=Pro_(ind_tmp,1)./sum(Pro_(ind_tmp,1));
    Pro_(ind_tmp,1)=tmp_nor;    
end

TransOut=find(Info_m(:,7)==3);
Pro_(Info_m(TransOut,5))=1;

Subterm=[Asub Aenz]*log([Metab_;Pro_]);  %in log form

rVuni=zeros(nvuni_,1);
Econc=ones(nRevEnz-nin_,1);
Tconc=ones(nin_+nout_,1);
Kvec=zeros(nvuni_,1);
%For reversible reactions
for i=1:nRevRxns
    nsteps_=0.5*(Info_m(i,4)-Info_m(i,3)+1);
    stepInd=0.5*(Info_m(i,3)+1):1:0.5*Info_m(i,4);
    for j=1:nsteps_
        rtmp=Rref(stepInd(j),1);
        if rtmp==1 || rVnet(i,1)==0
            rVuni(Info_m(i,3)+2*j-2,1)=rVuni(Info_m(i,3)+2*j-3,1);
        else            
            rVuni(Info_m(i,3)+2*j-2,1)=rVnet(i,1)/(1-rtmp.^sign(rVnet(i,1)));
            rVuni(Info_m(i,3)+2*j-1,1)=-rVnet(i,1)+rVuni(Info_m(i,3)+2*j-2,1);
        end
        Kvec(Info_m(i,3)+2*j-2,1)=rVuni(Info_m(i,3)+2*j-2,1)/exp(Subterm(Info_m(i,3)+2*j-2,1));
        Kvec(Info_m(i,3)+2*j-1,1)=rVuni(Info_m(i,3)+2*j-1,1)/exp(Subterm(Info_m(i,3)+2*j-1,1));            
    end        
end

%For irreversible reactions of transportations
for i=1:length(TransOut)
    rVuni(Info_m(TransOut(i),3),1)=rVnet(TransOut(i),1); 
    Kvec(Info_m(TransOut(i),3),1)=rVuni(Info_m(TransOut(i),3),1); %since Subterm 
end
eout=nRevRxns+nout_;

%For inhibition
if eout<totalRxns
    for i=eout+1:1:totalRxns
        inhRxn=Info_m(i,1);
        if Info_m(i,7)==4  %competition
            alpha_=thres_*rand;
            tmp_f=alpha_*rVuni(Info_m(inhRxn,3),1);
            rVuni(Info_m(i,3),1)=tmp_f;
            rVuni(Info_m(i,4),1)=tmp_f;
            Kvec(Info_m(i,3),1)=tmp_f/exp(Subterm(Info_m(i,3),1)); 
            Kvec(Info_m(i,4),1)=tmp_f/exp(Subterm(Info_m(i,4),1));
        elseif Info_m(i,7)==5 %noncompetition
            alpha_=thres_*rand;
            tmp_f=alpha_*rVuni(Info_m(inhRxn,3)+2,1);
            rVuni(Info_m(i,3),1)=tmp_f;
            rVuni(Info_m(i,4),1)=tmp_f;
            Kvec(Info_m(i,3),1)=tmp_f/exp(Subterm(Info_m(i,3),1)); 
            Kvec(Info_m(i,4),1)=tmp_f/exp(Subterm(Info_m(i,4),1));
        else
            alpha_=thres_*rand(2,1);
            tmp_f=alpha_.*rVuni([Info_m(inhRxn,3);Info_m(inhRxn,3)+2],1);
        
            rVuni(Info_m(i,3),1)=tmp_f(1);
            rVuni(Info_m(i,3)+1,1)=tmp_f(1);
            rVuni(Info_m(i,3)+2,1)=tmp_f(2);
            rVuni(Info_m(i,3)+3,1)=tmp_f(2);
            Kvec(Info_m(i,3),1)=tmp_f(1)/exp(Subterm(Info_m(i,3),1)); 
            Kvec(Info_m(i,3)+1,1)=tmp_f(1)/exp(Subterm(Info_m(i,3)+1,1)); 
            Kvec(Info_m(i,3)+2,1)=tmp_f(2)/exp(Subterm(Info_m(i,3)+2,1)); 
            Kvec(Info_m(i,3)+3,1)=tmp_f(2)/exp(Subterm(Info_m(i,3)+3,1)); 
        end
    end
end
    

