function Rref=samplingR(Info_m,FElow,FEhigh,method_)

%Sampling reversibility under thermodynamic constraints
%Last revision June 06, 2008
%Linh Tran, UCLA

if nargin<4 || isempty(method_)
    method_=1;
end

totalRxns=size(Info_m,1);  %including the inhibition reaction
nvuni_=Info_m(totalRxns,4);
nin_=length(find(Info_m(:,7)==2));
nout_=length(find(Info_m(:,7)==3));
nNetRxns=length(FElow);
nInterEnz=nNetRxns-nin_-nout_;
nRevRxns=nNetRxns-nout_;


tmpE=[FElow,FEhigh];
tmpE=abs(tmpE);

bound_(:,1)=min(tmpE,[],2);
bound_(:,2)=max(tmpE,[],2);
Rref=zeros(0.5*(nvuni_+nout_),1);  
rcount=1;
if method_==1
    %scheme 1: sampling ln(Rij) having distribution similar to (skew to the right) beta
    %distribution, then converting to Rij 
    for i=1:nInterEnz+1:1:nRevRxns
        nsteps_=0.5*(Info_m(i,4)-Info_m(i,3)+1);
        d1=repmat(bound_(i,1),nsteps_,1);
        d2=repmat(bound_(i,2),nsteps_,1);
        y=1/nsteps_.*(-d1+(d1-d2).*(1-betarnd(4.5,1.5,nsteps_,1)));
        Rref(rcount:rcount+nsteps_-1,1)=exp(y);
        rcount=rcount+nsteps_;
    end
else
    %scheme 2: sampling Rij uniformly, and then rescale to satisfy the constraint
    for i=1:nInterEnz+1:1:nRevRxns
        nsteps_=0.5*(Info_m(i,4)-Info_m(i,3)+1);
        tmpR=rand(nsteps_,1);
        tmpR_log=log(tmpR);
        sumTmpLog=sum(tmpR_log);
        if sumTmpLog>-bound_(i,1)
            delta_=(sumTmpLog+bound_(i,1))/nsteps_;
            tmpR_log=tmpR_log-repmat(delta_,nsteps_,1);        
        elseif sumTmpLog<-bound_(i,1)
            tbound=-bound_(i,:)/sumTmpLog;
            t=tbound(1)+(tbound(2)-tbound(1))*rand;
            tmpR_log=t*tmpR_log;
        end
        Rref(rcount:rcount+nsteps_-1,1)=exp(tmpR_log);
        rcount=rcount+nsteps_;
    end
end

rcount=rcount+nout_;  %skip the outfluxes which have R=0;

if nNetRxns <= totalRxns
    Rref(rcount:end)=1;  %Reversibility of inhibition reach equilibrium R=1;
end

    
